# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector.apiclient
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    This module covers all necessary functionality for http communicatiton
    with our Backend System.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""

from hexonet.apiconnector.response import Response
from hexonet.apiconnector.responsetemplatemanager import ResponseTemplateManager as RTM
from hexonet.apiconnector.socketconfig import SocketConfig
from urllib.parse import quote, unquote, urlparse, urlencode
from urllib.request import urlopen, Request
import re
import copy
import platform

rtm = RTM()

ISPAPI_CONNECTION_URL_PROXY = "http://127.0.0.1/api/call.cgi"
ISPAPI_CONNECTION_URL = "https://api.ispapi.net/api/call.cgi"


class APIClient(object):

    def __init__(self):
        # API connection url
        self.setURL(ISPAPI_CONNECTION_URL)
        # Object covering API connection data
        self.__socketConfig = SocketConfig()
        # activity flag for debug mode
        self.__debugMode = False
        # API connection timeout setting
        self.__socketTimeout = 300000
        self.useLIVESystem()
        # user agent setting
        self.__ua = ""
        # additional connection settings
        self.__curlopts = {}

    def setProxy(self, proxy):
        """
        Set Proxy to use for API communication
        """
        if proxy == '':
            self.__curlopts.pop('PROXY', None)
        else:
            self.__curlopts["PROXY"] = proxy
        return self

    def getProxy(self):
        """
        Get Proxy configuration value for API communication
        """
        if "PROXY" in self.__curlopts:
            return self.__curlopts["PROXY"]
        return None

    def setReferer(self, referer):
        """
        Set the Referer Header to use for API communication
        """
        if referer == '':
            self.__curlopts.pop('REFERER', None)
        else:
            self.__curlopts["REFERER"] = referer
        return self

    def getReferer(self):
        """
        Get the Referer Header configuration value
        """
        if "REFERER" in self.__curlopts:
            return self.__curlopts["REFERER"]
        return None

    def enableDebugMode(self):
        """
        Enable Debug Output to STDOUT
        """
        self.__debugMode = True
        return self

    def disableDebugMode(self):
        """
        Disable Debug Output
        """
        self.__debugMode = False
        return self

    def getPOSTData(self, cmd):
        """
        Serialize given command for POST request including connection configuration data
        """
        data = self.__socketConfig.getPOSTData()
        tmp = ""
        if not isinstance(cmd, str):
            for key in sorted(cmd.keys()):
                if (cmd[key] is not None):
                    tmp += ("{0}={1}\n").format(key, re.sub('[\r\n]', '', str(cmd[key])))
        return ("{0}{1}={2}").format(data, quote('s_command'), quote(re.sub('\n$', '', tmp)))

    def getSession(self):
        """
        Get the API Session that is currently set
        """
        return self.__socketConfig.getSession()

    def getURL(self):
        """
        Get the API connection url that is currently set
        """
        return self.__socketURL

    def getUserAgent(self):
        """
        Get the User Agent
        """
        if (len(self.__ua) == 0):
            pid = "PYTHON-SDK"
            pyv = platform.python_version()
            pf = platform.system()
            arch = platform.architecture()[0]
            self.__ua = "%s (%s; %s; rv:%s) python/%s" % (pid, pf, arch, self.getVersion(), pyv)
        return self.__ua

    def setUserAgent(self, pid, rv):
        """
        Possibility to customize default user agent to fit your needs by given string and revision
        """
        pyv = platform.python_version()
        pf = platform.system()
        arch = platform.architecture()[0]
        self.__ua = "%s (%s; %s; rv:%s) python-sdk/%s python/%s" % (pid, pf, arch, rv, self.getVersion(), pyv)
        return self

    def getVersion(self):
        """
        Get the current module version
        """
        return "3.4.1"

    def saveSession(self, session):
        """
        Apply session data (session id and system entity) to given client request session
        """
        session["socketcfg"] = {
            "entity": self.__socketConfig.getSystemEntity(),
            "session": self.__socketConfig.getSession()
        }
        return self

    def reuseSession(self, session):
        """
        Use existing configuration out of session
        to rebuild and reuse connection settings
        """
        self.__socketConfig.setSystemEntity(session["socketcfg"]["entity"])
        self.setSession(session["socketcfg"]["session"])
        return self

    def setURL(self, value):
        """
        Set another connection url to be used for API communication
        """
        self.__socketURL = value
        return self

    def setOTP(self, value):
        """
        Set one time password to be used for API communication
        """
        self.__socketConfig.setOTP(value)
        return self

    def setSession(self, value):
        """
        Set an API session id to be used for API communication
        """
        self.__socketConfig.setSession(value)
        return self

    def setRemoteIPAddress(self, value):
        """
        Set an Remote IP Address to be used for API communication.
        To be used in case you have an active ip filter setting.
        """
        self.__socketConfig.setRemoteAddress(value)
        return self

    def setCredentials(self, uid, pw):
        """
        Set Credentials to be used for API communication
        """
        self.__socketConfig.setLogin(uid)
        self.__socketConfig.setPassword(pw)
        return self

    def setRoleCredentials(self, uid, role, pw):
        """
        Set Credentials to be used for API communication
        """
        if (role == ''):
            return self.setCredentials(uid, pw)
        return self.setCredentials(("{0}!{1}").format(uid, role), pw)

    def login(self, otp=""):
        """
        Perform API login to start session-based communication
        """
        self.setOTP(otp)
        rr = self.request({"COMMAND": "StartSession"})
        if (rr.isSuccess()):
            col = rr.getColumn("SESSION")
            self.setSession(col.getData()[0] if (col is not None) else None)
        return rr

    def loginExtended(self, params, otp=""):
        """
        Perform API login to start session-based communication.
        Use given specific command parameters.
        """
        self.setOTP(otp)
        cmd = {
            "COMMAND": "StartSession"
        }
        cmd.update(params)
        rr = self.request(cmd)
        if (rr.isSuccess()):
            col = rr.getColumn("SESSION")
            self.setSession(col.getData()[0] if (col is not None) else None)
        return rr

    def logout(self):
        """
        Perform API logout to close API session in use
        """
        rr = self.request({
            "COMMAND": "EndSession",
        })
        if (rr.isSuccess()):
            self.setSession(None)
        return rr

    def request(self, cmd):
        """
        Perform API request using the given command
        """
        # flatten nested api command bulk parameters
        newcmd = self.__flattenCommand(cmd)
        # auto convert umlaut names to punycode
        newcmd = self.__autoIDNConvert(newcmd)

        # request command to API
        data = self.getPOSTData(newcmd).encode('UTF-8')
        # TODO: 300s (to be sure to get an API response)
        try:
            headers = {
                'User-Agent': self.getUserAgent()
            }
            if "REFERER" in self.__curlopts:
                headers['Referer'] = self.__curlopts["REFERER"]
            req = Request(self.__socketURL, data, headers)
            if "PROXY" in self.__curlopts:
                proxyurl = urlparse(self.__curlopts["PROXY"])
                req.set_proxy(proxyurl.netloc, proxyurl.scheme)
            body = urlopen(req, timeout=self.__socketTimeout).read()
            if (self.__debugMode):
                print((self.__socketURL, data, body, '\n', '\n'))
        except Exception:
            body = rtm.getTemplate("httperror").getPlain()
            if (self.__debugMode):
                print((self.__socketURL, data, "HTTP communication failed", body, '\n', '\n'))
        return Response(body, newcmd)

    def requestNextResponsePage(self, rr):
        """
        Request the next page of list entries for the current list query
        Useful for tables
        """
        mycmd = rr.getCommand()
        if ("LAST" in mycmd):
            raise Exception("Parameter LAST in use. Please remove it to avoid issues in requestNextPage.")
        first = 0
        if ("FIRST" in mycmd):
            first = int(mycmd["FIRST"])
        total = rr.getRecordsTotalCount()
        limit = rr.getRecordsLimitation()
        first += limit
        if (first < total):
            mycmd["FIRST"] = first
            mycmd["LIMIT"] = limit
            return self.request(mycmd)
        else:
            return None

    def requestAllResponsePages(self, cmd):
        """
        Request all pages/entries for the given query command
        """
        responses = []
        mycmd = copy.deepcopy(cmd)
        mycmd["FIRST"] = 0
        rr = self.request(mycmd)
        tmp = rr
        while tmp is not None:
            responses.append(tmp)
            tmp = self.requestNextResponsePage(tmp)
            if tmp is None:
                break
        return responses

    def setUserView(self, uid):
        """
        Set a data view to a given subuser
        """
        self.__socketConfig.setUser(uid)
        return self

    def resetUserView(self):
        """
        Reset data view back from subuser to user
        """
        self.__socketConfig.setUser(None)
        return self

    def useHighPerformanceConnectionSetup(self):
        """
        Activate High Performance Setup
        """
        self.setURL(ISPAPI_CONNECTION_URL_PROXY)
        return self

    def useDefaultConnectionSetup(self):
        """
        Activate Default Connection Setup (which is the default anyways)
        """
        self.setURL(ISPAPI_CONNECTION_URL)
        return self

    def useOTESystem(self):
        """
        Set OT&E System for API communication
        """
        self.__socketConfig.setSystemEntity("1234")
        return self

    def useLIVESystem(self):
        """
        Set LIVE System for API communication (this is the default setting)
        """
        self.__socketConfig.setSystemEntity("54cd")
        return self

    def __flattenCommand(self, cmd):
        """
        Flatten API command to handle it easier later on (nested array for bulk params)
        """
        newcmd = {}
        for key in list(cmd.keys()):
            newKey = key.upper()
            val = cmd[key]
            if val is None:
                continue
            if isinstance(val, list):
                i = 0
                while i < len(val):
                    newcmd[newKey + str(i)] = re.sub(r'[\r\n]', '', str(val[i]))
                    i += 1
            else:
                newcmd[newKey] = re.sub(r'[\r\n]', '', str(val))
        return newcmd

    def __autoIDNConvert(self, cmd):
        """
        Auto convert API command parameters to punycode, if necessary.
        """
        # don't convert for convertidn command to avoid endless loop
        # and ignore commands in string format(even deprecated)
        if isinstance(cmd, str) or re.match(r'^CONVERTIDN$', cmd["COMMAND"], re.IGNORECASE):
            return cmd

        toconvert = []
        keys = []
        for key in cmd:
            if re.match(r'^(DOMAIN|NAMESERVER|DNSZONE)([0-9]*)$', key, re.IGNORECASE):
                keys.append(key)
        if not keys.count:
            return cmd
        idxs = []
        for key in keys:
            if not re.match(r'^[a-z0-9.-]+$', cmd[key], re.IGNORECASE):
                toconvert.append(cmd[key])
                idxs.append(key)

        r = self.request({
            "COMMAND": "ConvertIDN",
            "DOMAIN": toconvert
        })
        if r.isSuccess():
            col = r.getColumn("ACE")
            if col is not None:
                for idx, pc in enumerate(col.getData()):
                    cmd[idxs[idx]] = pc
        return cmd



# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector.record
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    This module covers all necessary functionality to
    work with a record and wrapped data.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""


class Record(object):
    """
    The Record class covers all you need to access record data of a Backend API response.
    """

    def __init__(self, data):
        self.__data = data

    def getData(self):
        """
        get row data
        """
        return self.__data

    def getDataByKey(self, key):
        """
        get row data for given column name
        """
        if (self.__hasData(key)):
            return self.__data[key]
        return None

    def __hasData(self, key):
        """
        check if record has data for given column name
        """
        return key in self.__data



# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector.responsetemplatemanager
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    This module covers all functionality to
    manage response templates.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""

import hexonet.apiconnector.responseparser as RP
from hexonet.apiconnector.responsetemplate import ResponseTemplate as RT


class ResponseTemplateManager(object):
    """
    The ResponseTemplateManager class covers all functionality required to manage
    Response Templates.
    """
    # to keep the singleton instance
    __instance = None
    __templates = None

    def __new__(self):
        if ResponseTemplateManager.__instance is None:
            ResponseTemplateManager.__instance = object.__new__(self)
        rtm = ResponseTemplateManager.__instance

        ResponseTemplateManager.__templates = {
            "404": rtm.generateTemplate("421", "Page not found"),
            "500": rtm.generateTemplate("500", "Internal server error"),
            "empty": rtm.generateTemplate("423", "Empty API response. Probably unreachable API end point"),
            "error": rtm.generateTemplate("421", "Command failed due to server error. Client should try again"),
            "expired": rtm.generateTemplate("530", "SESSION NOT FOUND"),
            "httperror": rtm.generateTemplate("421", "Command failed due to HTTP communication error"),
            "invalid": rtm.generateTemplate("423", "Invalid API response. Contact Support"),
            "unauthorized": rtm.generateTemplate("530", "Unauthorized"),
        }
        return rtm

    @staticmethod
    def getInstance():
        """
        Returns the singleton instance
        """
        return ResponseTemplateManager()

    def generateTemplate(self, code, description):
        """
        Returns a response template string for the given code and description
        """
        return ('[RESPONSE]\r\nCODE={0}\r\nDESCRIPTION={1}\r\nEOF\r\n').format(code, description)

    def addTemplate(self, id, plain):
        """
        Add response template to template container
        """
        self.__templates[id] = plain
        return self.__instance

    def getTemplate(self, id):
        """
        Get response template instance from template container
        """
        if (self.hasTemplate(id)):
            return RT(self.__templates[id])
        return RT(self.generateTemplate("500", "Response Template not found"))

    def getTemplates(self):
        """
        Return all available response templates
        """
        tpls = {}
        for key in list(self.__templates.keys()):
            tpls[key] = RT(self.__templates[key])
        return tpls

    def hasTemplate(self, id):
        """
        Check if given template exists in template container
        """
        return (id in self.__templates)

    def isTemplateMatchHash(self, tpl2, id):
        """
        Check if given API response hash matches a given template by code and description
        """
        h = self.getTemplate(id).getHash()
        return (
            (h["CODE"] == tpl2["CODE"]) and
            (h["DESCRIPTION"] == tpl2["DESCRIPTION"])
        )

    def isTemplateMatchPlain(self, plain, id):
        """
        Check if given API plain response matches a given template by code and description
        """
        h = self.getTemplate(id).getHash()
        tpl2 = RP.parse(plain)
        return (
            (h["CODE"] == tpl2["CODE"]) and
            (h["DESCRIPTION"] == tpl2["DESCRIPTION"])
        )



# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector.socketconfig
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    This module covers all necessary functionality to
    pre-configure http communication with our Backend System.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""

from urllib.parse import quote, unquote


class SocketConfig(object):
    """
    The SocketConfig class helps to configure the http communcation with our Backend System.
    """

    def __init__(self):
        # API system entity. "54cd" for LIVE system; "1234" for OT&E system
        self.__entity = None
        # account name
        self.__login = None
        # one time password (2FA)
        self.__otp = None
        # account password
        self.__pw = None
        # remote ip address (ip filter)
        self.__remoteaddr = None
        # API session id
        self.__session = None
        # subuser account name (subuser specific data view)
        self.__user = None

    def getPOSTData(self):
        """
        Create POST data string out of connection data
        """
        data = ""
        tpl = "{0}={1}&"
        for key in ['__entity', '__login', '__otp', '__pw', '__remoteaddr', '__session', '__user']:
            item = getattr(self, '_SocketConfig' + key)
            if (item is not None and item is not ""):
                data += tpl.format(quote('s_' + key[2:]), quote(item))
        return data

    def getSession(self):
        """
        Get API Session ID in use
        """
        return self.__session

    def getSystemEntity(self):
        """
        Get API System Entity in use
        """
        return self.__entity

    def setLogin(self, value):
        """
        Set account name to use
        """
        self.__session = None
        self.__login = value
        return self

    def setOTP(self, value):
        """
        Set one time password to use
        """
        self.__session = None
        self.__otp = value
        return self

    def setPassword(self, value):
        """
        Set account password to use
        """
        self.__session = None
        self.__pw = value
        return self

    def setRemoteAddress(self, value):
        """
        Set Remote IP Address to use
        """
        self.__remoteaddr = value
        return self

    def setSession(self, value):
        """
        Set API Session ID to use
        """
        self.__login = None
        self.__pw = None
        self.__otp = None
        self.__session = value
        return self

    def setSystemEntity(self, value):
        """
        Set API System Entity to use. This is set to 54cd / LIVE System by default.
        """
        self.__entity = value
        return self

    def setUser(self, value):
        """
        Set subuser account name (for subuser data view)
        """
        self.__user = value
        return self



# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector.response
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    This module covers all necessary functionality to
    work with Backend API responses.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""
from hexonet.apiconnector.responsetemplate import ResponseTemplate as RT
from hexonet.apiconnector.column import Column
from hexonet.apiconnector.record import Record

import math


class Response(RT, object):
    """
    The Response class covers all necessary functionality to cover access to
    Backend API response data in a useful way.
    """

    def __init__(self, raw, cmd=None):
        super(Response, self).__init__(raw)
        # The API Command used within this request
        self.__command = cmd
        # Column names available in this responsse.
        # NOTE: this includes also FIRST, LAST, LIMIT, COUNT, TOTAL
        # and maybe further specific columns in case of a list query
        self.__columnkeys = []
        # Container of Column Instances
        self.__columns = []
        # Record Index we currently point to in record list
        self.__recordIndex = 0
        # Record List (List of rows)
        self.__records = []

        h = self.getHash()
        if ("PROPERTY" in h):
            colKeys = list(h["PROPERTY"].keys())
            count = 0
            for c in colKeys:
                d = h["PROPERTY"][c]
                self.addColumn(c, d)
                mylen = len(d)
                if (mylen > count):
                    count = mylen
            for i in range(count):
                d = {}
                for k in colKeys:
                    col = self.getColumn(k)
                    if (col is not None):
                        v = col.getDataByIndex(i)
                        if (v is not None):
                            d[k] = v
                self.addRecord(d)

    def addColumn(self, key, data):
        """
        Add a column to the column list
        """
        col = Column(key, data)
        self.__columns.append(col)
        self.__columnkeys.append(key)
        return self

    def addRecord(self, h):
        """
        Add a record to the record list
        """
        self.__records.append(Record(h))
        return self

    def getColumn(self, key):
        """
        Get column by column name
        """
        if (self.__hasColumn(key)):
            return self.__columns[self.__columnkeys.index(key)]
        return None

    def getColumnIndex(self, colkey, index):
        """
        Get Data by Column Name and Index
        """
        col = self.getColumn(colkey)
        return col.getDataByIndex(index) if (col is not None) else None

    def getColumnKeys(self):
        """
        Get Column Names
        """
        return self.__columnkeys

    def getColumns(self):
        """
        Get List of Columns
        """
        return self.__columns

    def getCommand(self):
        """
        Get Command used in this request
        """
        return self.__command

    def getCurrentPageNumber(self):
        """
        Get Page Number of current List Query
        """
        first = self.getFirstRecordIndex()
        limit = self.getRecordsLimitation()
        if (first is not None) and (limit):
            return math.floor(first / limit) + 1
        return None

    def getCurrentRecord(self):
        """
        Get Record of current record index
        """
        return self.__records[self.__recordIndex] if (self.__hasCurrentRecord()) else None

    def getFirstRecordIndex(self):
        """
        Get Index of first row in this response
        """
        col = self.getColumn("FIRST")
        if (col is not None):
            f = col.getDataByIndex(0)
            if (f is not None):
                return int(f)
        if (len(self.__records)):
            return 0
        return None

    def getLastRecordIndex(self):
        """
        Get last record index of the current list query
        """
        col = self.getColumn("LAST")
        if (col is not None):
            data = col.getDataByIndex(0)
            if (data is not None):
                return int(data)
        len = self.getRecordsCount()
        if (len):
            return (len - 1)
        return None

    def getListHash(self):
        """
        Get Response as List Hash including useful meta data for tables
        """
        lh = []
        for rec in self.getRecords():
            lh.append(rec.getData())
        return {
            "LIST": lh,
            "meta": {
                "columns": self.getColumnKeys(),
                "pg": self.getPagination()
            }
        }

    def getNextRecord(self):
        """
        Get next record in record list
        """
        if (self.__hasNextRecord()):
            self.__recordIndex += 1
            return self.__records[self.__recordIndex]
        return None

    def getNextPageNumber(self):
        """
        Get Page Number of next list query
        """
        cp = self.getCurrentPageNumber()
        if (cp is None):
            return None
        page = cp + 1
        pages = self.getNumberOfPages()
        return page if (page <= pages) else pages

    def getNumberOfPages(self):
        """
        Get the number of pages available for this list query
        """
        t = self.getRecordsTotalCount()
        limit = self.getRecordsLimitation()
        if (t and limit):
            return math.ceil(t / self.getRecordsLimitation())
        return 0

    def getPagination(self):
        """
        Get object containing all paging data
        """
        return {
            "COUNT": self.getRecordsCount(),
            "CURRENTPAGE": self.getCurrentPageNumber(),
            "FIRST": self.getFirstRecordIndex(),
            "LAST": self.getLastRecordIndex(),
            "LIMIT": self.getRecordsLimitation(),
            "NEXTPAGE": self.getNextPageNumber(),
            "PAGES": self.getNumberOfPages(),
            "PREVIOUSPAGE": self.getPreviousPageNumber(),
            "TOTAL": self.getRecordsTotalCount()
        }

    def getPreviousPageNumber(self):
        """
        Get Page Number of previous list query
        """
        cp = self.getCurrentPageNumber()
        if (cp is not None):
            cp = cp - 1
            if (cp):
                return cp
        return None

    def getPreviousRecord(self):
        """
        Get previous record in record list
        """
        if (self.__hasPreviousRecord()):
            self.__recordIndex -= 1
            return self.__records[self.__recordIndex]
        return None

    def getRecord(self, idx):
        """
        Get Record at given index
        """
        if (idx >= 0 and len(self.__records) > idx):
            return self.__records[idx]
        return None

    def getRecords(self):
        """
        Get all Records
        """
        return self.__records

    def getRecordsCount(self):
        """
        Get count of rows in this response
        """
        return len(self.__records)

    def getRecordsTotalCount(self):
        """
        Get total count of records available for the list query
        """
        col = self.getColumn("TOTAL")
        if (col is not None):
            t = col.getDataByIndex(0)
            if (t is not None):
                return int(t)
        return self.getRecordsCount()

    def getRecordsLimitation(self):
        """
        Get limit(ation) setting of the current list query
        """
        col = self.getColumn("LIMIT")
        if (col is not None):
            data = col.getDataByIndex(0)
            if (data is not None):
                return int(data)
        return self.getRecordsCount()

    def hasNextPage(self):
        """
        Check if this list query has a next page
        """
        cp = self.getCurrentPageNumber()
        if (cp is None):
            return False
        return ((cp + 1) <= self.getNumberOfPages())

    def hasPreviousPage(self):
        """
        Check if this list query has a previous page
        """
        cp = self.getCurrentPageNumber()
        if (cp is None):
            return False
        return ((cp - 1) > 0)

    def rewindRecordList(self):
        """
        Reset index in record list back to zero
        """
        self.__recordIndex = 0
        return self

    def __hasColumn(self, key):
        """
        Check if column exists in response
        """
        try:
            self.__columnkeys.index(key)
        except ValueError:
            return False
        return True

    def __hasCurrentRecord(self):
        """
        Check if the record list contains a record for the
        current record index in use
        """
        tlen = len(self.__records)
        return (
            tlen > 0 and
            self.__recordIndex >= 0 and
            self.__recordIndex < tlen
        )

    def __hasNextRecord(self):
        """
        Check if the record list contains a next record for the
        current record index in use
        """
        next = self.__recordIndex + 1
        return (self.__hasCurrentRecord() and (next < len(self.__records)))

    def __hasPreviousRecord(self):
        """
        Check if the record list contains a previous record for the
        current record index in use
        """
        return (self.__recordIndex > 0 and self.__hasCurrentRecord())



# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector.column
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    This module covers all necessary functionality to
    work with a column and wrapped data.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""


class Column(object):
    """
    The Column class covers all you need to access column data of a Backend API response.
    """

    def __init__(self, key, data):
        self.__key = key
        self.__data = data
        self.length = len(data)

    def getKey(self):
        """
        Get column name
        """
        return self.__key

    def getData(self):
        """
        Get column data
        """
        return self.__data

    def getDataByIndex(self, idx):
        """
        Get column data at given index
        """
        return self.__data[idx] if self.hasDataIndex(idx) else None

    def hasDataIndex(self, idx):
        """
        Check if column has a given data index
        """
        return True if ((idx >= 0) and (idx < self.length)) else False
